/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.core.datagram;

import io.vertx.reactivex.RxHelper;
import io.vertx.reactivex.ObservableHelper;
import io.vertx.reactivex.FlowableHelper;
import io.vertx.reactivex.impl.AsyncResultMaybe;
import io.vertx.reactivex.impl.AsyncResultSingle;
import io.vertx.reactivex.impl.AsyncResultCompletable;
import io.vertx.reactivex.WriteStreamObserver;
import io.vertx.reactivex.WriteStreamSubscriber;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * A received datagram packet (UDP) which contains the data and information about the sender of the data itself.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.core.datagram.DatagramPacket original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.core.datagram.DatagramPacket.class)
public class DatagramPacket {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    DatagramPacket that = (DatagramPacket) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<DatagramPacket> __TYPE_ARG = new TypeArg<>(    obj -> new DatagramPacket((io.vertx.core.datagram.DatagramPacket) obj),
    DatagramPacket::getDelegate
  );

  private final io.vertx.core.datagram.DatagramPacket delegate;
  
  public DatagramPacket(io.vertx.core.datagram.DatagramPacket delegate) {
    this.delegate = delegate;
  }

  public DatagramPacket(Object delegate) {
    this.delegate = (io.vertx.core.datagram.DatagramPacket)delegate;
  }

  public io.vertx.core.datagram.DatagramPacket getDelegate() {
    return delegate;
  }

  /**
   * Returns the {@link io.vertx.reactivex.core.net.SocketAddress} of the sender that sent
   * this {@link io.vertx.reactivex.core.datagram.DatagramPacket}.
   * @return the address of the sender
   */
  public io.vertx.reactivex.core.net.SocketAddress sender() { 
    io.vertx.reactivex.core.net.SocketAddress ret = io.vertx.reactivex.core.net.SocketAddress.newInstance((io.vertx.core.net.SocketAddress)delegate.sender());
    return ret;
  }

  /**
   * Returns the data of the {@link io.vertx.reactivex.core.datagram.DatagramPacket}
   * @return the data
   */
  public io.vertx.reactivex.core.buffer.Buffer data() { 
    io.vertx.reactivex.core.buffer.Buffer ret = io.vertx.reactivex.core.buffer.Buffer.newInstance((io.vertx.core.buffer.Buffer)delegate.data());
    return ret;
  }

  public static DatagramPacket newInstance(io.vertx.core.datagram.DatagramPacket arg) {
    return arg != null ? new DatagramPacket(arg) : null;
  }

}
