/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.ext.auth.htpasswd;

import io.vertx.reactivex.RxHelper;
import io.vertx.reactivex.ObservableHelper;
import io.vertx.reactivex.FlowableHelper;
import io.vertx.reactivex.impl.AsyncResultMaybe;
import io.vertx.reactivex.impl.AsyncResultSingle;
import io.vertx.reactivex.impl.AsyncResultCompletable;
import io.vertx.reactivex.WriteStreamObserver;
import io.vertx.reactivex.WriteStreamSubscriber;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * An extension of AuthProvider which is using htpasswd file as store
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.auth.htpasswd.HtpasswdAuth original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.auth.htpasswd.HtpasswdAuth.class)
public class HtpasswdAuth extends io.vertx.reactivex.ext.auth.authentication.AuthenticationProvider {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    HtpasswdAuth that = (HtpasswdAuth) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<HtpasswdAuth> __TYPE_ARG = new TypeArg<>(    obj -> new HtpasswdAuth((io.vertx.ext.auth.htpasswd.HtpasswdAuth) obj),
    HtpasswdAuth::getDelegate
  );

  private final io.vertx.ext.auth.htpasswd.HtpasswdAuth delegate;
  
  public HtpasswdAuth(io.vertx.ext.auth.htpasswd.HtpasswdAuth delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public HtpasswdAuth(Object delegate) {
    super((io.vertx.ext.auth.htpasswd.HtpasswdAuth)delegate);
    this.delegate = (io.vertx.ext.auth.htpasswd.HtpasswdAuth)delegate;
  }

  public io.vertx.ext.auth.htpasswd.HtpasswdAuth getDelegate() {
    return delegate;
  }

  public static io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth create(io.vertx.reactivex.core.Vertx vertx) { 
    io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth ret = io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth.newInstance((io.vertx.ext.auth.htpasswd.HtpasswdAuth)io.vertx.ext.auth.htpasswd.HtpasswdAuth.create(vertx.getDelegate()));
    return ret;
  }

  public static io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth create(io.vertx.reactivex.core.Vertx vertx, io.vertx.ext.auth.htpasswd.HtpasswdAuthOptions htpasswdAuthOptions) { 
    io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth ret = io.vertx.reactivex.ext.auth.htpasswd.HtpasswdAuth.newInstance((io.vertx.ext.auth.htpasswd.HtpasswdAuth)io.vertx.ext.auth.htpasswd.HtpasswdAuth.create(vertx.getDelegate(), htpasswdAuthOptions));
    return ret;
  }

  public static HtpasswdAuth newInstance(io.vertx.ext.auth.htpasswd.HtpasswdAuth arg) {
    return arg != null ? new HtpasswdAuth(arg) : null;
  }

}
