/*
 * Copyright 2014 Red Hat, Inc.
 *
 * Red Hat licenses this file to you under the Apache License, version 2.0
 * (the "License"); you may not use this file except in compliance with the
 * License.  You may obtain a copy of the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */

package io.vertx.reactivex.ext.auth.sqlclient;

import io.vertx.reactivex.RxHelper;
import io.vertx.reactivex.ObservableHelper;
import io.vertx.reactivex.FlowableHelper;
import io.vertx.reactivex.impl.AsyncResultMaybe;
import io.vertx.reactivex.impl.AsyncResultSingle;
import io.vertx.reactivex.impl.AsyncResultCompletable;
import io.vertx.reactivex.WriteStreamObserver;
import io.vertx.reactivex.WriteStreamSubscriber;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Iterator;
import java.util.function.Function;
import java.util.stream.Collectors;
import io.vertx.core.Handler;
import io.vertx.core.AsyncResult;
import io.vertx.core.json.JsonObject;
import io.vertx.core.json.JsonArray;
import io.vertx.lang.rx.RxGen;
import io.vertx.lang.rx.TypeArg;
import io.vertx.lang.rx.MappingIterator;

/**
 * Factory interface for creating  instances that use the Vert.x SQL client.
 *
 * <p/>
 * NOTE: This class has been automatically generated from the {@link io.vertx.ext.auth.sqlclient.SqlAuthorization original} non RX-ified interface using Vert.x codegen.
 */

@RxGen(io.vertx.ext.auth.sqlclient.SqlAuthorization.class)
public class SqlAuthorization extends io.vertx.reactivex.ext.auth.authorization.AuthorizationProvider {

  @Override
  public String toString() {
    return delegate.toString();
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) return true;
    if (o == null || getClass() != o.getClass()) return false;
    SqlAuthorization that = (SqlAuthorization) o;
    return delegate.equals(that.delegate);
  }
  
  @Override
  public int hashCode() {
    return delegate.hashCode();
  }

  public static final TypeArg<SqlAuthorization> __TYPE_ARG = new TypeArg<>(    obj -> new SqlAuthorization((io.vertx.ext.auth.sqlclient.SqlAuthorization) obj),
    SqlAuthorization::getDelegate
  );

  private final io.vertx.ext.auth.sqlclient.SqlAuthorization delegate;
  
  public SqlAuthorization(io.vertx.ext.auth.sqlclient.SqlAuthorization delegate) {
    super(delegate);
    this.delegate = delegate;
  }

  public SqlAuthorization(Object delegate) {
    super((io.vertx.ext.auth.sqlclient.SqlAuthorization)delegate);
    this.delegate = (io.vertx.ext.auth.sqlclient.SqlAuthorization)delegate;
  }

  public io.vertx.ext.auth.sqlclient.SqlAuthorization getDelegate() {
    return delegate;
  }

  /**
   * Create a JDBC authorization provider implementation
   * @param client the SQL client instance
   * @return the auth provider
   */
  public static io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization create(io.vertx.reactivex.sqlclient.SqlClient client) { 
    io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization ret = io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization.newInstance((io.vertx.ext.auth.sqlclient.SqlAuthorization)io.vertx.ext.auth.sqlclient.SqlAuthorization.create(client.getDelegate()));
    return ret;
  }

  /**
   * Create a JDBC authorization provider implementation
   * @param client the SQL client instance
   * @param options the {@link io.vertx.ext.auth.sqlclient.SqlAuthorizationOptions}
   * @return the auth provider
   */
  public static io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization create(io.vertx.reactivex.sqlclient.SqlClient client, io.vertx.ext.auth.sqlclient.SqlAuthorizationOptions options) { 
    io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization ret = io.vertx.reactivex.ext.auth.sqlclient.SqlAuthorization.newInstance((io.vertx.ext.auth.sqlclient.SqlAuthorization)io.vertx.ext.auth.sqlclient.SqlAuthorization.create(client.getDelegate(), options));
    return ret;
  }

  public static SqlAuthorization newInstance(io.vertx.ext.auth.sqlclient.SqlAuthorization arg) {
    return arg != null ? new SqlAuthorization(arg) : null;
  }

}
